#This software is a rerelease of ArrayFiltering.R module of Microarray
#Expression Potential Suite with alterations in the input and ouput format in
#order to improve transition between modules of eMESS.

#No part of this text or software may be used or reproduced in any matter
#whatsoever without written permission, except in the case of brief quotations
#embodied in critical articles of or for reviews.  For information address
#Brian W. Kirk (bwkbem@yahoo.com).
#Software Copyright (C) October 2002  Brian W. Kirk
#Form/Text Copyright (C) October 2002  Brian W. Kirk
#Released for use with R v1.5.1 (C)

#Copyright (C) October 2002  Brian W. Kirk
#Released for R v1.5.1 (C)

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.


###############################################################################
#                                FUNCTIONS                                    #
###############################################################################

#FUNCTION THAT PERFORMS A T-TEST WITH SUMMARY DATA, FOR TWO SAMPLES WITH
#UNEQUAL VARIANCES (IE GREATER THAN 2-FOLD DIFFERENCE).  USING SATTHERWAITE
#APPROXIMATION. 

  fstudent.t.test <- function(Mu1=m1, Mu2=m2, SD1=stand.dev1, SD2=stand.dev2,
                              n1=num1, n2=num2, sides=2)
   {
    t <- (Mu1-Mu2)/sqrt((SD1^2/n1)+(SD2^2/n2))
    degrees.freedom <- round(  (SD1^2/n1 + SD2^2/n2)^2 /
                     ((SD1^2)/(n1^2*(n1-1)) + (SD2^2)/(n2^2*(n2-1))));


    p <- sides * (1 - pt(abs(t), degrees.freedom));
  }


##FUNCTION THAT CREATES A VIRTUAL ARRAY BASED ON A PASSED FILTER AND WRITES IT
##TO A FILE
  virtual.array <- function(XCoord=x, YCoord=y, Filter=filt,
                                   OutFilePrefix=out, Title="Filter")
   {
   postscript(file=paste(OutFilePrefix, "VA.eps", sep=""), horizontal=FALSE,
             onefile=TRUE);

   par(bg="black");

   Filter <- as.logical(Filter);
   x0 <- XCoord[Filter];
   y0 <- YCoord[Filter];

   plot(y0 ~ x0, type="p", pch=16, xlab="", ylab = "", cex= 0.1,
        xlim=c(min(XCoord), max(XCoord)),
        ylim=c(min(YCoord), max(YCoord)),
        col="purple");
   mtext(Title, side=3, col="white", adj=0.5);

   dev.off()
   }


###############################################################################
#                           CONSTANT DECLARATION                              #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x", "y", or "z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANENCY

                ############################################
                ######## DEFINE PERMANENT CONSTANTS ########
                ############################################


#Define Input and Output Files
RawData <- paste(InputDirectory, InputFile, sep ="");

OutputPrefix <- paste(OutputDirectory, OutputFile, sep="");


             #####################################################
             ########### DEFINE TEMPORARY CONSTANTS ##############
             #####################################################

#Define Column Position For Input File
xBlockPos <- 1;
xColumnPos <- 2;
xRowPos <- 3;
xIDPos <- 4
xXCoordPos <- 5;
xYCoordPos <- 6;
xMedF1Pos <- 7;           
xSDF1Pos <- 8;
xMedB1Pos <- 9;           
xSDB1Pos <- 10;
xMedF2Pos <- 11;          
xSDF2Pos <- 12;
xMedB2Pos <- 13;          
xSDB2Pos <- 14;
xFPixelPos <- 15;
xBPixelPos <- 16;
xFlagPos <- 17;       

###############################################################################
#                             VARIABLE DECLARATION                            #
###############################################################################

#CONSTANTS AND VARIABLES OF TEMPORARY USE BEGIN WITH "x", "y", or "z" AND ALL
#USER DEFINED FUNCTIONS BEGIN WITH "f" or "g" IN ORDER TO FACILITATE REMOVABLE
#FROM MEMORY.  WHEN USING VARIABLES WITH THESE RESERVED INTIAL LETTERS,
#NEAREST CLEANUP SHOULD BE IDENTIFIED TO INSURE PROPER PERMANCY
   
               ################################################
               ######### CREATE VECTORS FROM mgpr.txt #########
               ################################################

#INPUT MODIFIED GPR GILE (mgpr.txt) AND EXTRACT PERTINENT COLUMNS
xMGPR <- data.frame(read.delim(RawData));


#SORT MGPR BY BLOCK, COLUMN AND THEN ROW ALL IN ASCENDING ORDER
zBlockVec <- as.numeric(as.vector(xMGPR[, xBlockPos]));
zColumnVec <- as.numeric(as.vector(xMGPR[, xColumnPos]));
zRowVec <- as.numeric(as.vector(xMGPR[, xRowPos]));

zSortFilter <- order(zBlockVec, zRowVec, zColumnVec);

#EXTRACT PERTINENT COLUMNS FROM MGPR AND ASSIGN EACH TO A UNIQUE VECTOR IN
#SORTED FORM

zBlockVec <- as.numeric(as.vector(zBlockVec[zSortFilter]));
zColumnVec <- as.numeric(as.vector(zColumnVec[zSortFilter]));
zRowVec <- as.numeric(as.vector(zRowVec[zSortFilter]));
zIDVec <- as.character(as.vector(xMGPR[zSortFilter, xIDPos]));
XCoordVec <- as.numeric(as.vector(xMGPR[zSortFilter, xXCoordPos]));
YCoordVec <- as.numeric(as.vector(xMGPR[zSortFilter, xYCoordPos]));
zF1Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xMedF1Pos]));
zSDF1Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xSDF1Pos]));
zB1Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xMedB1Pos]));
zSDB1Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xSDB1Pos]));
zF2Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xMedF2Pos]));
zSDF2Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xSDF2Pos]));
zB2Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xMedB2Pos]));
zSDB2Vec <- as.numeric(as.vector(xMGPR[zSortFilter, xSDB2Pos]));
zFPixelVec <- as.numeric(as.vector(xMGPR[zSortFilter, xFPixelPos]));
zBPixelVec <- as.numeric(as.vector(xMGPR[zSortFilter, xBPixelPos]));
zFlagVec <- as.numeric(as.vector(xMGPR[zSortFilter, xFlagPos]));


#REMOVE TEMPORARY VARIABLES/CONSTANTS
rm(list=ls(pat="^x"));

###############################################################################
#                               MAIN BODY                                     #
###############################################################################

#Set Initial MasterFilter to all TRUE and prime first column of FilterTable
DefaultFilter <- rep(1, length(zIDVec));
MasterFilter <- DefaultFilter;
FilterTable <- DefaultFilter;

                       ############################
                       ##### CREATE FILTERS #######
                       ############################

#FLAG PROBES AS DEFINED BY AN EXTERNAL FILTER AS BEING SUBOPTIMAL
if (UseExternal) {
  ExternalFilter <- rep(0, length(zIDVec));
  ExternalFilter[which(zFlagVec==1)] <- 1;
  MasterFilter <- MasterFilter * ExternalFilter;
  FilterTable <- cbind(FilterTable, ExternalFilter);
  ExternalCount <- length(ExternalFilter) - sum(ExternalFilter); 
  if (VirtualArray) {
    OutFile <- paste(OutputPrefix, "Ext", sep="");
    virtual.array(XCoordVec, YCoordVec, ExternalFilter, OutFile,
                  "External Filter")
  }
} else {
  ExternalCount <- 0;
  FilterTable <- cbind(FilterTable, DefaultFilter)
}


#FLAG PROBES AS DEFINED BY THE USER BASED ON ID AS BEING SUBOPTIMAL
if (UseID) {
  IDFilter <- as.numeric(!as.logical(match(zIDVec, IDCrit, nomatch=0)));
  FilterTable <- cbind(FilterTable, IDFilter);
  MasterFilter <- MasterFilter * IDFilter;
  IDCount <- length(IDFilter) - sum(IDFilter);
  if (VirtualArray) {
    OutFile <- paste(OutputPrefix, "ID", sep="");
    virtual.array(XCoordVec, YCoordVec, IDFilter, OutFile,
                  "ID Filter")
  }
} else {
  IDCount <- 0;
  FilterTable <- cbind(FilterTable, DefaultFilter)
}



#FLAG PROBES THAT ARE SATURATED (DEFINED BY MaxIntensity)
if (UseSaturation) {
  SaturationFilter <- DefaultFilter;
  SaturationFilter[which((zF1Vec >= MaxIntensity) |
                         (zF2Vec >= MaxIntensity))] <- 0;
  MasterFilter <- MasterFilter * SaturationFilter;
  FilterTable<- cbind(FilterTable, SaturationFilter);
  SaturationCount <- length(SaturationFilter) - sum(SaturationFilter);
  if (VirtualArray) {
    OutFile <- paste(OutputPrefix, "Sat", sep="");
    virtual.array(XCoordVec, YCoordVec, SaturationFilter, OutFile,
                  "Saturation Filter")
  }
} else {
  SaturationCount <- 0;
  FilterTable <- cbind(FilterTable, DefaultFilter)
}

 
#FLAG LOW INTENSITY PROBES BASED ON A LOWER SIGNAL LIMIT
if (UseFloor) {
  FloorFilter <- DefaultFilter;

  #Flag probes that fail in either channels.  This is applied to MasterFilter 
  FloorFilter[which(((zF1Vec <= MinIntensity)|(zF1Vec/zB1Vec <= MinSigtoBG))|
              ((zF2Vec <= MinIntensity)|(zF2Vec/zB2Vec <= MinSigtoBG)))] <- 0;
  MasterFilter <- MasterFilter * FloorFilter;
  FilterTable <- cbind(FilterTable, FloorFilter);
  FloorCount <- length(FloorFilter) - sum(FloorFilter);
  if (VirtualArray) {
    OutFile <- paste(OutputPrefix, "Floor", sep="");
    virtual.array(XCoordVec, YCoordVec, FloorFilter, OutFile,
                  "Floor Filter")
  }

  #Flag probes that fail in only one channel.  This is for user reference.
  #This result is NOT APPLIED TO MasterFilter
  FloorSplit <- as.numeric(!FloorFilter &
                !(((zF1Vec <= MinIntensity)|(zF1Vec/zB1Vec <= MinSigtoBG))&
                 ((zF2Vec <= MinIntensity)|(zF2Vec/zB2Vec <= MinSigtoBG))))
} else {
  FloorCount <- 0;
  FilterTable <- cbind(FilterTable, DefaultFilter)
}



#FLAG PROBES WITH BACKGROUND GREATER THAN FOREGROUND OR SIGNAL
#INDISTINGUISHABLE FROM NOISE AS DETERMINED BY A ONE-SIDED T-TEST WITH
#A BONFERRONI CORRECTION (ALPHA/NUMBER OF PROBES) FOR MULTIPLE SAMPLES
if (UsetTest) {
  xp1.value <- fstudent.t.test(zF1Vec, zB1Vec, zSDF1Vec, zSDB1Vec, zFPixelVec,
                            zBPixelVec, 1); 
  xp2.value <- fstudent.t.test(zF2Vec, zB2Vec, zSDF2Vec, zSDB2Vec, zFPixelVec,
                            zBPixelVec, 1);

  #Bonferroni Correction
  xCorrection <- length(zIDVec); 

  p1.value <- xp1.value * xCorrection;
  p2.value <- xp2.value * xCorrection;

  p1.value[which(p1.value > 1)] <- 1;
  p2.value[which(p2.value > 1)] <- 1;
  
  #Flag probes that fail in either channels.  This is applied to MasterFilter 
  tTestFilter <- DefaultFilter;

  tTestFilter[which(((zB1Vec >= zF1Vec)|(p1.value  > Alpha))|
                   ((zB2Vec >= zF2Vec)|(p2.value  > Alpha)))] <- 0;
  MasterFilter <- MasterFilter * tTestFilter;
  FilterTable <- cbind(FilterTable, tTestFilter);
  tTestCount <- length(tTestFilter) - sum(tTestFilter);
  if (VirtualArray) {
    OutFile <- paste(OutputPrefix, "tTest", sep="");
    virtual.array(XCoordVec, YCoordVec, tTestFilter, OutFile,
                  "tTest Filter")
  }
  
  #Flag probes that fail in only one channel.  This is for user reference.
  #This result is NOT APPLIED TO MasterFilter
  tTestSplit <- as.numeric(
                  !tTestFilter & !(((zB1Vec >= zF1Vec)|(p1.value  > Alpha)) &
                                   ((zB2Vec >= zF2Vec)|(p2.value  > Alpha))));
} else {
  tTestCount <- 0;
  FilterTable <- cbind(FilterTable, DefaultFilter)
}


#Assign MasterFilter to FilterTable
FilterTable[,1] <- MasterFilter;

if (VirtualArray) {
  OutFile <- paste(OutputPrefix, "Master", sep="");
  virtual.array(XCoordVec, YCoordVec, MasterFilter, OutFile,
                "Master Filter")
}

MasterCountRmn <- sum(MasterFilter);
MasterCountRmv <- length(MasterFilter)-MasterCountRmn;
  
#REMOVE TEMPORARY VARIABLES/CONSTANTS
rm(list=ls(pat="^x"));
rm(list=ls(pat="^f"));

##############################################################################
#                              OUTPUT RESULTS                                #
##############################################################################

###### OuputTable for Lowess
if (BGSubtract) {
  SampleSignal <- zF1Vec-zB1Vec;
  RefSignal <- zF2Vec-zB2Vec
} else {
  SampleSignal <- zF1Vec;
  RefSignal <- zF2Vec
}

SignalTable <- cbind(zBlockVec, zColumnVec, zRowVec, zIDVec, MasterFilter,
                     SampleSignal, RefSignal);

SignalHeader1 <- paste(RawData);
SignalHeader2 <- date();
SignalHeader3 <- c(paste("UseExternal = ", as.character(UseExternal), sep=""),
                   paste("ProbesRemoved = ", as.character(ExternalCount),
                         sep=""));
SignalHeader4 <- c(paste("UseID = ", as.character(UseID), sep=""),
                   paste("ProbesRemoved = ", as.character(IDCount),
                         sep=""),
                   paste("IDCrit = ", as.character(IDCrit), sep=""));
SignalHeader5 <- c(paste("UseSaturation = ", as.character(UseSaturation),
                         sep=""),
                   paste("ProbesRemoved = ", as.character(SaturationCount),
                         sep=""),
                   paste("MaxIntensity = ", as.character(MaxIntensity),
                         sep=""));
SignalHeader6 <- c(paste("UseFloor = ", as.character(UseFloor), sep=""),
                   paste("ProbesRemoved = ", as.character(FloorCount),
                         sep=""),
                   paste("MinSigtoBG = ", as.character(MinSigtoBG),
                         sep=""),
                   paste("MinIntensity = ", as.character(MinIntensity), sep=""));
SignalHeader7 <- c(paste("UsetTest = ", as.character(UsetTest), sep=""),
                   paste("ProbesRemoved = ", as.character(tTestCount),
                         sep=""),
                   paste("Alpha = ", as.character(Alpha), sep=""));
SignalHeader8 <- c(paste("Background Subtract = ", as.character(BGSubtract),
                         sep=""));
SignalHeader9 <- c(paste("CombinedProbesRemain = ",
                         as.character(MasterCountRmn), sep=""),
                   paste("CombinedProbesRemoved = ",
                         as.character(MasterCountRmv), sep=""));
SignalHeaderTable <-  c("Block", "Column", "Row", "ID", "MasterFilter",
                        "Samp_Signal", "Ref_Signal");

cat(SignalHeader1, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n");
cat(SignalHeader2, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader3, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader4, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader5, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader6, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader7, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader8, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeader9, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
cat(SignalHeaderTable, file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".PInt", sep=""),
      sep="\n", append=TRUE);
  
write.table(SignalTable, file=paste(OutputPrefix, ".PInt", sep=""),
      quote=FALSE, sep="\t", col.names=FALSE, row.names=FALSE, append=TRUE);


#### Filter Table
#list block, column, row, ID, All FIlters
FilterTable <- cbind(zBlockVec, zColumnVec, zRowVec, zIDVec, FilterTable);

FilterHeader1 <- paste(RawData);
FilterHeader2 <- date();
FilterHeader3 <- c(paste("UseExternal = ", as.character(UseExternal), sep=""),
                   paste("ProbesRemoved = ", as.character(ExternalCount),
                         sep=""));
FilterHeader4 <- c(paste("UseID = ", as.character(UseID), sep=""),
                   paste("ProbesRemoved = ", as.character(IDCount),
                         sep=""),
                   paste("IDCrit = ", as.character(IDCrit), sep=""));
FilterHeader5 <- c(paste("UseSaturation = ", as.character(UseSaturation),
                         sep=""),
                   paste("ProbesRemoved = ", as.character(SaturationCount),
                         sep=""),
                   paste("MaxIntensity = ", as.character(MaxIntensity),
                         sep=""));
FilterHeader6 <- c(paste("UseFloor = ", as.character(UseFloor), sep=""),
                   paste("ProbesRemoved = ", as.character(FloorCount),
                         sep=""),
                   paste("MinSigtoBG = ", as.character(MinSigtoBG),
                         sep=""),
                   paste("MinIntensity = ", as.character(MinIntensity), sep=""));
FilterHeader7 <- c(paste("UsetTest = ", as.character(UsetTest), sep=""),
                   paste("ProbesRemoved = ", as.character(tTestCount),
                         sep=""),
                   paste("Alpha = ", as.character(Alpha), sep=""));
FilterHeader8 <- c(paste("CombinedProbesRemain = ",
                         as.character(MasterCountRmn), sep=""),
                   paste("CombinedProbesRemoved = ",
                         as.character(MasterCountRmv), sep=""));
FilterHeaderFill <- rep(" ", 1);
FilterHeaderTable <-  c("Block", "Column", "Row", "ID", "MasterFilter",
                        "ExtFilter", "IDFilter", "SatFilter", "FloorFilter",
                        "tTestFilter");

cat(FilterHeader1, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n");
cat(FilterHeader2, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader3, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader4, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader5, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader6, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader7, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeader8, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderFill, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
cat(FilterHeaderTable, file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\t", append=TRUE);
cat(c(""), file=paste(OutputPrefix, ".Filter", sep=""),
      sep="\n", append=TRUE);
  
write.table(FilterTable, file=paste(OutputPrefix, ".Filter", sep=""),
      quote=FALSE, sep="\t", col.names=FALSE, row.names=FALSE, append=TRUE);

#### Split
# test for UsetTest and UseFloor and process accord

#Combine FloorSplit and tTestSplit
if (UseFloor & UsetTest) {
  NoSplit <- FALSE; 
  ComboSplit <- as.numeric(FloorSplit | tTestSplit);
} else {
  NoSplit <- TRUE; 
  ComboSplit <- rep(NA, length(zIDVec))
}

if (NoSplit) {
  Filler <- rep(NA, length(zIDVec));
  if (UseFloor) {
    tTestSplit <- Filler;
    p1.value <- Filler;
    p2.value <- Filler;
    Alpha <- NA
    NoSplit <- FALSE;
  }
  if (UsetTest) {
    FloorSplit <- Filler;
    NoSplit <- FALSE;
  }  
}

if (!NoSplit) {
  SplitTable <- cbind(zBlockVec, zColumnVec, zRowVec, zIDVec, FloorSplit,
                    tTestSplit, ComboSplit, zF1Vec, zB1Vec, p1.value,
                    zF2Vec, zB2Vec, p2.value);
  SplitHeader1 <- paste(RawData);
  SplitHeader2 <- date();
  SplitHeader3 <- c(paste("UseFloor = ", as.character(UseFloor), sep=""),
                    paste("MinSigtoBG = ", as.character(MinSigtoBG), sep=""),
                    paste("MinIntensity = ", as.character(MinIntensity), sep=""));
  SplitHeader4 <- c(paste("UsetTest = ", as.character(UsetTest), sep=""),
                    paste("Alpha = ", as.character(Alpha), sep=""));
  SplitHeaderFill <- rep(" ", 5);
  SplitHeaderTable <-  c("Block", "Column", "Row", "ID", "FloorSplit",
                    "tTestSplit", "ComboSplit", "F_Samp", "B_Samp",
                    "pVal_Samp", "F_Ref", "B_Ref", "pVal_Ref");


  cat(SplitHeader1, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n");
  cat(SplitHeader2, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n", append=TRUE);
  cat(SplitHeader3, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\t", append=TRUE);
  cat(c(""), file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n", append=TRUE);
  cat(SplitHeader4, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\t", append=TRUE);
  cat(c(""), file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n", append=TRUE);
  cat(SplitHeaderFill, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n", append=TRUE);
  cat(SplitHeaderTable, file=paste(OutputPrefix, ".Split", sep=""),
      sep="\t", append=TRUE);
  cat(c(""), file=paste(OutputPrefix, ".Split", sep=""),
      sep="\n", append=TRUE);
  
  write.table(SplitTable, file=paste(OutputPrefix, ".Split", sep=""),
      quote=FALSE, sep="\t", col.names=FALSE, row.names=FALSE, append=TRUE);
}

rm(list=ls());
